/* Author: Dave Weissman  dweisman@remus.rutgers.edu  */


import java.io.*;
import java.util.*;


public class FileWorldGenerator{
    

    public static void setUpDomain(int numfolders, int fc, int pc, int gr){
	try{
	    System.out.println("(define (domain file-world)");
	    System.out.println("\t(:requirements\t:disjunctive-preconditions :negative-preconditions");
	    System.out.print("\t\t:conditional-effects ");
	    if((fc != 0) || (pc != 0) || (gr != 0)){
		System.out.print(":rewards");
	    }
	    System.out.println();
	    System.out.println("\t\t:probabilistic-effects :universal-preconditions)");
	    System.out.println("\t(:predicates\t(has-type ?p)");
	    System.out.println("\t\t(goes-in ?p ?f)");
	    System.out.println("\t\t(filed ?p)");
	    for(int i = 0; i < numfolders; i++){
		System.out.println("\t\t(haveF" + i + ")");
	    }//for
	    System.out.println("\t\t)");
	    System.out.print("\t(:constants ");
	    for(int i = 0; i < numfolders; i++){
		System.out.print("F" + i + " ");
	    }//for
	    System.out.println(")");
	    System.out.println();
	}//try
	catch(Exception e){
	    System.out.println(e + " occured in setUpDomain");
	    System.exit(0);
	}//catch
    }//setUpDomain

    public static void setUpTyper(int numfolders, int pre){
	
	double numfold = (double) numfolders;
	double prob = 1 / (numfold);
	int scaleprob = (int) Math.round(1000*prob);
	int scalerest = 1000 - ((numfolders - 1)*scaleprob);
	prob = ((double) scaleprob)/1000;
	double rest = ((double) scalerest)/1000;
	
	try{
	    System.out.println("(:action get-type");
	    System.out.println("\t:parameters (?p)");
	    if(pre == 1){
		System.out.println("\t:precondition\t(and (not (has-type ?p)))");
	    }
	    System.out.println("\t:effect");
	    if(pre == 0){
		System.out.println("(when (not (has-type ?p))");
	    }
	    System.out.println("(and (has-type ?p)");
	    System.out.println("\t\t(probabilistic");
	    for(int i = 0; i < numfolders - 1; i++){
		System.out.println("\t\t\t" + prob + "\t(goes-in ?p F" + i + ")");
	    }//for
	    System.out.println("\t\t\t" + rest + "\t(goes-in ?p F" + (numfolders-1) + ")");
	    System.out.println("))");
	    if(pre == 0){
		System.out.print(")");
	    }
	    System.out.println(")");
	    System.out.println();
	}//try
	catch(Exception e){
	    System.out.println(e + " occured in setUpTyper");
	    System.exit(0);
	}//catch
    }//setUpTyper

    
    public static void setUpGetFolder(int foldernum, int numfolders, int fc, int pre){
	try{
	    System.out.println("(:action get-folder-F" + foldernum);
	    if(pre == 1){
		System.out.print("\t:precondition  (and ");
		for(int i = 0; i < numfolders; i++){
		    System.out.println("\t\t(not (haveF" + i + ")) ");
		}//for
		System.out.println(")");
	    }
	    System.out.print("\t:effect ");
	    if(fc != 0){
		System.out.print("(and (decrease (reward) " + (0-fc) + ")");
	    }
	    if(pre == 0){
		System.out.println("(when (and ");
		for(int i = 0; i < numfolders; i++){
		    System.out.println("\t\t(not (haveF" + i + ")) ");
		}
		System.out.println(")");
	    }
	    System.out.println();
	    System.out.println("(haveF" + foldernum + ")");
	    if(pre == 0){
		System.out.print(")");
	    }
	    if(fc != 0){
		System.out.print(")");
	    }
	    System.out.println(")");
	    System.out.println();
	}//try
	catch(Exception e){
	    System.out.println(e + " occured in setUpGetFolder");
	    System.exit(0);
	}//catch
    }//setUpGetFolder

    public static void setUpFile(int foldernum, int pc, int pre){
	try{
	    System.out.println("(:action file-F" + foldernum);
	    System.out.println("\t:parameters (?p)");
	    if(pre == 1){
		System.out.println("\t:precondition (and (haveF" + foldernum + ") (has-type ?p) (goes-in ?p F" + foldernum + "))");
	    }
	    System.out.print("\t:effect ");
	    if(pc != 0){
		System.out.print("(and (decrease (reward) " + (0-pc) + ")");
	    }
	    if(pre == 0){
		System.out.println("(when (and (haveF" + foldernum + ") (has-type ?p) (goes-in ?p F" + foldernum + "))");
	    }
	    System.out.println();
	    System.out.println("(filed ?p)");
	    if(pre == 0){
		System.out.print(")");
	    }
	    if(pc != 0){
		System.out.print(")");
	    }
	    System.out.println(")");
	    System.out.println();
	}//try
	catch(Exception e){
	    System.out.println(e + " occured in setUpFile");
	    System.exit(0);
	    }//catch
    }//setUpFile

    public static void setUpReturn(int foldernum, int pre){
	try{
	    System.out.println("(:action return-F" + foldernum);
	    if(pre == 1){
		System.out.println("\t:precondition\t(haveF" + foldernum + ")");
	    }
	    System.out.println("\t:effect");
	    if(pre == 0){
		System.out.println("(when (haveF" + foldernum + ")");
	    }
	    System.out.println("\t(not (haveF" + foldernum + "))");
	    if(pre == 0){
		System.out.print(")");
	    }
	    System.out.println(")");
	    System.out.println();
	}//try
	catch(Exception e){
	    System.out.println(e + " occured in setUpReturn");
	    System.exit(0);
	}//catch
    }//setUpReturn

    public static void setUpProblem(int numfiles, int numfolders, int greward){
	try{
	    System.out.println("(define (problem file-prob)");
	    System.out.println("\t(:domain file-world)"); 
	    System.out.print("\t(:objects ");
	    for(int i = 0; i < numfiles; i++){
		System.out.print("p" + i + " ");
	    }//for
	    System.out.println(")");
	    System.out.print("\t(:goal (and ");
	    for(int i = 0; i < numfiles; i++){
		System.out.print("(filed p" + i + ") ");
	    }//for
	    System.out.println("))");
	    if(greward != 0){
		System.out.println("(:goal-reward " + greward + ")");
	    }
	    System.out.println(")");
	}//try
	catch(Exception e){
	    System.out.println(e + " occured in setUpProblem");
	    System.exit(0);
	}//catch
    }//setUpProblem

    public static void myusage(){
	System.out.println("USAGE: [-nf number-of-folders] [-np number-of-files]");
	System.out.println("USAGE: [-fc get-folder-cost] [-pc filing-cost] [-gr goal-reward]");
	System.out.println("USAGE: [-pre precondition-indicator]");
	
	System.out.println("USAGE: number-of-folders must be a positive integer");
	System.out.println("USAGE: number-of-files must be a positive integer");
	System.out.println("USAGE: get-folder-cost must be a nonpositive integer");
	System.out.println("USAGE: filing-cost must be a nonpositive integer");
	System.out.println("USAGE: goal-reward must be a natural number");
	System.out.println("USAGE: precondition-indicator must be boolean valued");
	
	
    }


    public static void main(String[] args){
	int numfolders = 3;
	int numfiles = 10;
	int greward = 0;
	int foldcost = 0;
	int filecost = 0;
	int pre = 1;
	try{
	    for(int i = 0; i < args.length; i++){
		if((args[i].compareTo("-nf")) == 0){
		    if(i == args.length - 1){
			myusage();
			System.exit(0);
		    }
		    else{
			numfolders = Integer.parseInt(args[++i]);
			if(numfolders <= 0){
			    myusage();
			    System.exit(0);
			}
		    }
		}
		if((args[i].compareTo("-fc")) == 0){
		    if(i == args.length - 1){
			myusage();
			System.exit(0);
		    }
		    else{
			foldcost = Integer.parseInt(args[++i]);
			if(foldcost > 0){
			    myusage();
			    System.exit(0);
			}
		    }
		}
		if((args[i].compareTo("-pre")) == 0){
		    if(i == args.length){
			myusage();
			System.exit(0);
		    }
		    else{
			pre = Integer.parseInt(args[++i]);
			if(pre != 0 && pre != 1){
			    System.out.println("Problem with pre parameter");
			    System.exit(0);
			}
		    }
		}
		if((args[i].compareTo("-pc")) == 0){
		    if(i == args.length -1){
			myusage();
			System.exit(0);
		    }
		    else{
			filecost = Integer.parseInt(args[++i]);
			if(filecost > 0){
			    myusage();
			    System.exit(0);
			}
		    }
		}
		if((args[i].compareTo("-np")) == 0){
		    if(i == args.length - 1){
			myusage();
			System.exit(0);
		    }
		    else{
			numfiles = Integer.parseInt(args[++i]);
			if(numfiles <=0){
			    myusage();
			    System.exit(0);
			}
			
		    }
		}
		if((args[i].compareTo("-gr")) == 0){
		    if(i == args.length - 1){
			myusage();
			System.exit(0);
		    }
		    else{
			greward = Integer.parseInt(args[++i]);
			if(greward < 0){
			    myusage();
			    System.exit(0);
			}
		    }
		}   
	    }
	    System.out.println(";;;  This domain has been generated with the fileworld generator ;;;");
	    System.out.println(";;;      Created by Dave Weissman   (dnweissman@yahoo.com)       ;;;");
	    System.out.println("\n\n\n");
	    setUpDomain(numfolders, foldcost, filecost, greward);
	    setUpTyper(numfolders, pre);
	    for(int j = 0; j < numfolders; j++){
		setUpGetFolder(j, numfolders, foldcost, pre);
	    }//for
	    for(int j = 0; j < numfolders; j++){
		setUpFile(j, filecost, pre);
	    }//for
	    for(int j = 0; j < numfolders; j++){
		setUpReturn(j, pre);
	    }//for
	    System.out.println(")");	    
	    setUpProblem(numfiles, numfolders, greward);
	}//try
	catch(Exception e){
	    System.out.println(e);
	    System.exit(0);
	}//catch
    }//else
}//FWG









